#!/usr/bin/env node

/**
 * source文件夹格式为：
 * --src
 *   --widgets  //存放控件
 *     --select
 *       --index.js
 *     --input
 *       --index.js
 *     --radio
 *       --index.js
 *   --static  //存放需要移入到打包文件夹的静态文件
 *   --layout  //存放layout模板文件
 *       index.pug
 */

const jsonfile = require('jsonfile');
const _ = require('lodash');
const fs = require('fs');
const pug = require('pug');
const path = require('path');
const program = require('commander');

program
    .version('0.0.1', '-v, --version')
    .usage('[options] <file ...>')
    .option('-i, --input [path]', 'Json文件或者文件夹路径')
    .option('-o, --output [path]', '文件输入路径')
    .option('-s, --source [path]', '模板文件和静态文件文件夹路径');

program.on('--help', function () {
    console.log('');
    console.log('Example:');
    console.log('  $ j2f ./example/jsondir --source ./example/source --output ./example/build');
    console.log('');
    console.log('source文件夹格式为:');
    console.log('                      ├── layout                // 存放layout模板文件');
    console.log('                      │   └── index.pug         // layout文件');
    console.log('                      ├── static                // 存放需要移入到打包文件夹的静态文件');
    console.log('                      │   └── style.css');
    console.log('                      │   └── bootstrap.js');
    console.log('                      └── widgets               // 存放控件的文件夹');
    console.log('                          ├── label_input       // 每一个控件文件夹中需要有一个index.js文件');
    console.log('                          │   ├── index.js      // index.js需要实现compile方法');
    console.log('                          │   └── label_input.pug');
    console.log('                          └── label_select');
    console.log('                              ├── index.js');
    console.log('                              └── label_select.pug');

});

program.parse(process.argv);

/**路径处理 */
// 读取参数文件路径，默认第一个参数为文件路径
if (!program.input && program.args.length == 0) {
    throw '请输入json文件路径或文件夹路径';
}
if (!program.source) {
    throw '请输入资源路径';
}

// 以命令执行路径为基准，将路径转换为绝对路径。
const _jsonPath = program.input ? path.resolve(program.input) : path.resolve(program.args[0]);

// 资源路径
const _sourcePath = path.resolve(program.source);

// 输出路径
const _outputPath = program.output ? path.resolve(program.output) : path.resolve('./') + '/build';

// 获取layout路径
const _layoutPath = path.join(_sourcePath, 'layout/index.pug');

try {
    const layoutStat = fs.statSync(_layoutPath);
    if (!layoutStat.isFile()) {
        throw `${_layoutPath}不是一个pug文件`;
    }
} catch (e) {
    throw `layout文件夹错误，无法找到layout/index.pug，${e}`;
}

// 存放所有的widgets
const _widgets = {};

function main() {
    // 获取widgets
    const widgetsPath = path.join(_sourcePath, 'widgets');
    try {
        const widgetsStat = fs.statSync(widgetsPath);
        if (!widgetsStat.isDirectory()) {
            throw 'widgets不是一个文件夹';
        }
    } catch (e) {
        throw `${_sourcePath}中没有找到widgets路径，请按格式放入`;
    }

    let widgetDirs;
    try {
        widgetDirs = fs.readdirSync(widgetsPath);
    } catch (e) {
        throw `${widgetsPath}路径错误，${e}`;
    }
    // 遍历没一个widget文件夹
    for (let i = 0; i < widgetDirs.length; i++) {
        const dirName = widgetDirs[i];
        const dirPath = path.join(widgetsPath, dirName);
        try {
            const dirStat = fs.statSync(dirPath);
            if (!dirStat.isDirectory()) {
                // 不是widgets文件夹直接忽略
                continue;
            }
        } catch (e) {
            throw `${dirName}文件夹出错：${e}`;
        }
        const indexPath = path.join(dirPath, 'index.js');
        try {
            const indexStat = fs.statSync(indexPath);
            if (!indexStat.isFile()) {
                throw `${indexPath}不是一个文件`;
            }
        } catch (e) {
            throw `没有找到${indexPath}`;
        }

        _widgets[dirName] = require(indexPath);
    }

    // 判断输出路径文件夹是否存在，不存在就创建
    try {
        const outputStat = fs.statSync(_outputPath);
        if (outputStat.isFile()) {
            throw `输出路径错误，${_outputPath}是一个文件`;
        }
    } catch (e) {
        fs.mkdir(_outputPath, (err)=>{
            if (err) {
                throw `创建输出文件夹失败,\n${_outputPath}\n${err}`;
            }
        });
    }

    /**处理流程 */

    // 判断输入的是文件夹还是文件
    let fileStat;
    try {
        fileStat = fs.statSync(_jsonPath);
    } catch (e) {
        throw `json文件或文件夹路径错误，jsonPath == ${_jsonPath}`;
    }
    if (fileStat.isFile()) {
        const filetype = path.extname(_jsonPath);
        if (filetype == '.json') {
            const params = compileJson(_jsonPath);
            buildFile(params);
        } else {
            throw '输入文件格式不是json';
        }
    } else if (fileStat.isDirectory()) {
        const files = fs.readdirSync(_jsonPath);
        for (let i = 0; i < files.length; i++) {
            const file = files[i];
            if (path.extname(file) == '.json') {
                const filePath = path.join(_jsonPath, file);
                const params = compileJson(filePath);
                buildFile(params);
            }
        }
    }

    copyfiles(path.join(_sourcePath, 'static'), _outputPath);
    console.log(`解析完成，输出文件在\n${_outputPath}`);
}

// 将编出来的HTML组件拼到模板上并输出
function buildFile(params) {
    // 写出一个模板
    try {
        fs.writeFileSync(_outputPath + '/' + params.filename + '.html', pug.renderFile(_layoutPath, params));
    } catch (e) {
        throw `输出${params.filename}失败\n${e}`;
    }
}

// 解析Json文件，生成HTML组件
function compileJson(_jsonPath) {
    // 读取Json文件参数
    var j2fJson;
    try {
        j2fJson = jsonfile.readFileSync(_jsonPath); // 利用config构造pug可用参数
    } catch (e) {
        throw `json文件格式错误${e}`;
    }
    const filename = path.basename(_jsonPath).split('.')[0];
    var params = {};
    params.filename = filename;
    params.formTitle = j2fJson.formTitle;
    params.content = [];
    params.modals = [];
    const {
        content,
        modals
    } = j2fJson;
    console.log(`*****正在解析${filename}*****`);
    for (let index = 0; index < content.length; index++) {
        const section = content[index];
        let parsedSection = {};
        parsedSection.title = section.title;
        parsedSection.icon_class = section.icon_class;
        switch (section.type) {
            case 'form':
                parsedSection.first_content = parseWidgetList(section.first);
                parsedSection.second_content = parseWidgetList(section.second);
                parsedSection.third_content = parseWidgetList(section.third);
                parsedSection.type = 'form';

                break;
            case 'table':
                parsedSection.content = parseWidgetList(section.table);
                parsedSection.type = 'table';
                break;
            default:
                break;
        }
        params.content.push(parsedSection);
    }
    
    params.modals.push(parseWidgetList(modals));
    console.log(`*****${filename}解析完成*****`);
    return params;
}

// 将模板需要的js和css文件一起拷贝到输出文件夹
function copyfiles(source, output) {
    var staticStat;
    try {
        staticStat = fs.statSync(source);
    } catch (e) {
        return;
    }
    if (staticStat.isDirectory()) {
        const files = fs.readdirSync(source);
        for (var i = 0; i < files.length; i++) {
            const file = files[i];
            const originFile = path.join(source, file);
            const targetFile = path.join(output, file);
            fs.copyFileSync(originFile, targetFile);
        }
    } else if (staticStat.isFile()) {
        fs.copyFileSync(source, path.join(output, path.basename(source)));
    }
}

// 解析单个widget
function parseWidget(type, config) {
    const widget = _widgets[type];
    if (!widget) {
        throw `找不到${type}的解析函数`;
    }
    if (typeof widget.compile !== 'function') {
        throw `widget:${widget}中没有complie函数`;
    }
    let parsedW;
    try {
        parsedW = widget.compile(config);
    } catch (e) {
        throw `${type}解析出错:\n函数为parseWidget\n{\n type=${type} \n config=${config}\n}\n错误信息\n------\n${e}\n------`;
    }
    return parsedW;
}

// 解析一列widget
function parseWidgetList(list) {
    var parsedContent = '';
    if (list && list.length > 0) {
        for (let i = 0; i < list.length; i++) {
            const ele = list[i];
            parsedContent += parseWidget(ele.type, ele.options);
        }
    }
    return parsedContent;
}

main();